@@ -22,6 +22,10 @@
 # 20210302  GvB       bug when x is matrix: xx[1:seg_len] -> xx[1:seg_len, ]
 # 20210408  GvB       added range parameter ('half' o 'whole')
 # 20211021  GvB       corrected bug in col ptr into coh, phase, trans matrices
+# 20220405  GvB       remove padding to nearest power of 2 (Github Disc #6)
+#                     bug in returning vector when ncol is 1 (Github Issue #5)
+# 20220511  GvB       use inherits() instead of direct comparison of class name
+# 20220512  GvB       plot method for class 'pwelch'
 #------------------------------------------------------------------------------
 
 #' Welch’s power spectral density estimate
@@ -194,7 +198,7 @@ pwelch <- function(x, window = nextpow2(sqrt(NROW(x))), overlap = 0.5,
                    fs = 1,
                    detrend = c("long-mean", "short-mean",
                                "long-linear", "short-linear", "none"),
-                  range = if (is.numeric(x)) "half" else "whole") {
+                   range = if (is.numeric(x)) "half" else "whole") {
 
   # check parameters
   if (!(is.vector(x) || is.matrix(x)) &&
@@ -204,9 +208,11 @@ pwelch <- function(x, window = nextpow2(sqrt(NROW(x))), overlap = 0.5,
 
   series <- deparse(substitute(x))
   if (is.vector(x)) {
+    vec <- TRUE
     x <- as.matrix(x, ncol = 1)
     snames <- ""
   } else {
+    vec <- FALSE
     snames <- colnames(x)
   }
   x_len <- nrow(x)
@@ -247,7 +253,9 @@ pwelch <- function(x, window = nextpow2(sqrt(NROW(x))), overlap = 0.5,
   # initialize variables
   seg_len <- length(window)
   overlap <- trunc(seg_len * overlap)
-  nfft <- nextpow2(max(nfft, seg_len))
+  # GvB 20220405 removed nextpow2
+  # nfft <- nextpow2(max(nfft, seg_len))
+  nfft <- max(nfft, seg_len)
   win_meansq <- as.vector(window %*% window / seg_len)
   if (overlap >= seg_len) {
     stop("overlap must be smaller than windowlength")
@@ -365,7 +373,7 @@ pwelch <- function(x, window = nextpow2(sqrt(NROW(x))), overlap = 0.5,
   }
   freq <- seq.int(0, psd_len - 1) * (fs / nfft)
 
-  if (ns == 1) {
+  if (vec) {
     spec <- as.vector(spec)
   }
   all <- list(freq = freq, spec = spec, cross = cross, phase = phase,
@@ -375,16 +383,17 @@ pwelch <- function(x, window = nextpow2(sqrt(NROW(x))), overlap = 0.5,
   class(all) <- "pwelch"
   all
 }
+
 #' @rdname pwelch
 #' @export
 
-plot.pwelch <-
-  function(x, plot.type = c("spectrum", "cross-spectrum",
-                            "phase", "coherence", "transfer"),
-           yscale = c("linear", "log", "dB"),
-           xlab = NULL, ylab = NULL, main = NULL, ...) {
+plot.pwelch <- function(
+    x, xlab = NULL, ylab = NULL, main = NULL,
+    plot.type = c("spectrum", "cross-spectrum",
+                  "phase", "coherence", "transfer"),
+    yscale = c("linear", "log", "dB"), ...) {
 
-  if (!("pwelch" %in% class(x))) {
+  if (!inherits(x, "pwelch")) {
     stop("invalid object type")
   }
   plot.type <- match.arg(plot.type)
@@ -465,7 +474,7 @@ plot.pwelch <-
 
   }
   graphics::matplot(x$freq, plt, type = "l", xlab = xlab, ylab = "", ...)
-  graphics::title(main, sub = sub)
+  graphics::title(main = main, sub = sub)
   graphics::title(ylab = ylab, line = 2)
 
   if (plot.type == "phase") {
@@ -477,10 +486,13 @@ plot.pwelch <-
   }
 }
 
+#' @rdname pwelch
+#' @export
+
 print.pwelch <-
   function(x, plot.type = c("spectrum", "cross-spectrum",
                             "phase", "coherence", "transfer"),
            yscale = c("linear", "log", "dB"),
            xlab = NULL, ylab = NULL, main = NULL, ...) {
     plot.pwelch(x, plot.type, yscale, xlab, ylab, main, ...)
-}
\ No newline at end of file
+  }
